#include <stdio.h>
#include <stdlib.h>
#include <float.h>

#include <mef.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

int main(int argc, char **argv) {
  int k = 3;
  int n = 5000;
  int d = 1;
  double *data = malloc(k * n * sizeof(double));

  const gsl_rng_type *T;
  gsl_rng_env_setup();
  T = gsl_rng_default;
  gsl_rng *rng = gsl_rng_alloc(T);


  for (unsigned int i=0; i<n; i++) {
    data[i] = gsl_ran_gamma(rng, 1.0, 1/0.5);
  }
  for (unsigned int i=n; i<2*n; i++) {
    data[i] = gsl_ran_gamma(rng, 2.0, 1/0.5);
  }
  for (unsigned int i=2*n; i<3*n; i++) {
    data[i] = gsl_ran_gamma(rng, 9.0, 1/2.0);
  }

  emgamma* emgamma = emgamma_create(k, data, k*n, d);
  /* emgamma->maxiter = 10; */

  double best_error = DBL_MAX;
  kmeans* km;
  kmeans* km0;
  for (unsigned int i=0; i<10; i++) {
    km0 = kmeans_create(k, data, k*n, d);
    kmeans_initialize_random(km0, rng);
    kmeans_run(km0);

    if (km0->old_error < best_error) {
      km = km0;
      best_error = km0->old_error;
    }
    else {
      kmeans_destroy(km0);
    }
  }

  emgamma_initialize_from_clustering(emgamma, km->weights, km->affectation);
  
  mixture_fprint(stdout, emgamma->mixture, SOURCE);

  emgamma_run(emgamma);
  mixture_fprint(stdout, emgamma->mixture, SOURCE);

  kmeans_destroy(km);

  emgamma_destroy(emgamma);
}

